
/* 
 * Copyright (c) 1993-2003,2012 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*------------------  Fichier gnr automatiquement ------------------*/
/*------------------  Ne pas diter manuellement !!! ------------------*/

#ifndef p3d_MSG_LIB_H
#define p3d_MSG_LIB_H

#include "portLib.h"
#include "csLib.h"
#include "posterLib.h"
 
#include "p3dType.h"

#define TIME_WAIT_REPLY           /*2000*/ 0
 



/*----------- Tailles des messages et boites aux lettres  ----------*/

/* Taille du buffer utilise dans la bal pour un mesage de taille size
 * (cf servMbox pour UNIX et h2rgnLib pour VXWORKS )
 */


#define BUF_SIZE(size) \
   (size + 4 - (size & 3) + sizeof(LETTER_HDR) + 8)

/**
 ** Boite aux lettres reception requetes 
 **/
#define  P3D_MBOX_NAME                  genomInstanceName("p3d")

#define  P3D_MAX_RQST_SIZE              1032

#define  P3D_MBOX_RQST_SIZE       \
 (BUF_SIZE(P3D_MAX_RQST_SIZE) * SERV_NMAX_RQST_ID)

/** 
 ** Taille max repliques (serveur)
 **/
#define  P3D_MAX_INTERMED_REPLY_SIZE   (sizeof(int))

#define  P3D_MAX_REPLY_SIZE             264

/** 
 ** Taille bal clients pour reception repliques 
 **/
#define  P3D_CLIENT_MBOX_REPLY_SIZE   \
((BUF_SIZE(P3D_MAX_REPLY_SIZE) \
   + BUF_SIZE(P3D_MAX_INTERMED_REPLY_SIZE)) \
 * CLIENT_NMAX_RQST_ID)

/*-------------------------- Liste des requetes  -------------------------*/

#define P3D_INIT_RQST 0
#define P3D_GOTOSTOP_RQST 1
#define P3D_GOTO_RQST 2
#define P3D_GETLAMAGEOM_RQST 3
#define P3D_SETLAMAGEOM_RQST 4
#define P3D_GETLAMALIMITS_RQST 5
#define P3D_SETLAMALIMITS_RQST 6
#define P3D_GETARCSPARAMS_RQST 7
#define P3D_SETARCSPARAMS_RQST 8
#define P3D_SETGOALPARAMS_RQST 9
#define P3D_GETGOALPARAMS_RQST 10
#define P3D_ADDWAYPOINT_RQST 11
#define P3D_CLEARWAYPOINTS_RQST 12
#define P3D_CLEARCURRENTWAYPOINT_RQST 13
#define P3D_TRACK_RQST 14
#define P3D_TRACKSTOP_RQST 15
#define P3D_SETP3DPOSTER_RQST 16
#define P3D_READDEMP3DPOSTER_RQST 17
#define P3D_SAVEP3DMAP_RQST 18
#define P3D_PLACEROBOTONMAP_RQST 19
#define P3D_CONFIGMONITOR_RQST 20
#define P3D_STOPMONITOR_RQST 21
#define P3D_GETDUMPPARAMS_RQST 22
#define P3D_SETDUMPPARAMS_RQST 23
#define P3D_ABORT_RQST 24


/*---------------- PROTOTYPES DES FONCTIONS EXTERNES ------------------*/

#ifdef __cplusplus
extern "C" {
#endif

STATUS p3dClientInit (CLIENT_ID *pClientId);

STATUS p3dClientEnd (CLIENT_ID clientId);

int p3dAbortRqstSend (CLIENT_ID clientId, 
			   int *pRqstId,
			   int *activity,
			   int replyTimeOut);

int p3dAbortReplyRcv (CLIENT_ID clientId, 
			   int rqstId, 
			   int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			   int *bilan);

int p3dAbortRqstAndRcv (CLIENT_ID clientId, 
			     int *activity,
			     int *bilan);




extern STATUS p3dInitRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3D_INIT *(in_initParams),
					 int replyTimeOut);

extern int p3dInitReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dInitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3D_INIT *(in_initParams), 
					
					int *activity, int *bilan);

extern int p3dInitRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3D_INIT *(in_initParams), 
					
					int *activity,
					int *bilan);




extern STATUS p3dGoToStopRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dGoToStopReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int p3dGoToStopRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS p3dGoToRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3DWayPoint *(in_wpP),
					 int replyTimeOut);

extern int p3dGoToReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dGoToRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3DWayPoint *(in_wpP), 
					
					int *activity, int *bilan);

extern int p3dGoToRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3DWayPoint *(in_wpP), 
					
					int *activity,
					int *bilan);




extern STATUS p3dGetLamaGeomRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dGetLamaGeomReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       P3DLamaGeom *(out_geomP),
			       int *bilan);

int p3dGetLamaGeomRqstAndRcv (CLIENT_ID clientId, 
				  
				 P3DLamaGeom *(out_geomP),
				 int *bilan);




extern STATUS p3dSetLamaGeomRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3DLamaGeom *(in_geomP),
					 int replyTimeOut);

extern int p3dSetLamaGeomReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dSetLamaGeomRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3DLamaGeom *(in_geomP), 
					
					int *activity, int *bilan);

extern int p3dSetLamaGeomRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3DLamaGeom *(in_geomP), 
					
					int *activity,
					int *bilan);




extern STATUS p3dGetLamaLimitsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dGetLamaLimitsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       P3DLamaLimits *(out_limitsP),
			       int *bilan);

int p3dGetLamaLimitsRqstAndRcv (CLIENT_ID clientId, 
				  
				 P3DLamaLimits *(out_limitsP),
				 int *bilan);




extern STATUS p3dSetLamaLimitsRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3DLamaLimits *(in_limitsP),
					 int replyTimeOut);

extern int p3dSetLamaLimitsReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dSetLamaLimitsRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3DLamaLimits *(in_limitsP), 
					
					int *activity, int *bilan);

extern int p3dSetLamaLimitsRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3DLamaLimits *(in_limitsP), 
					
					int *activity,
					int *bilan);




extern STATUS p3dGetArcsParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dGetArcsParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       P3DArcsParams *(out_arcsP),
			       int *bilan);

int p3dGetArcsParamsRqstAndRcv (CLIENT_ID clientId, 
				  
				 P3DArcsParams *(out_arcsP),
				 int *bilan);




extern STATUS p3dSetArcsParamsRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3DArcsParams *(in_arcsP),
					 int replyTimeOut);

extern int p3dSetArcsParamsReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dSetArcsParamsRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3DArcsParams *(in_arcsP), 
					
					int *activity, int *bilan);

extern int p3dSetArcsParamsRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3DArcsParams *(in_arcsP), 
					
					int *activity,
					int *bilan);




extern STATUS p3dSetGoalParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 P3DPlanParams *(in_planP),
					 int replyTimeOut);

int p3dSetGoalParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int p3dSetGoalParamsRqstAndRcv (CLIENT_ID clientId, 
				 P3DPlanParams *(in_planP), 
				 
				 int *bilan);




extern STATUS p3dGetGoalParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dGetGoalParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       P3DPlanParams *(out_planP),
			       int *bilan);

int p3dGetGoalParamsRqstAndRcv (CLIENT_ID clientId, 
				  
				 P3DPlanParams *(out_planP),
				 int *bilan);




extern STATUS p3dAddWayPointRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 P3DWayPoint *(in_wpP),
					 int replyTimeOut);

int p3dAddWayPointReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int p3dAddWayPointRqstAndRcv (CLIENT_ID clientId, 
				 P3DWayPoint *(in_wpP), 
				 
				 int *bilan);




extern STATUS p3dClearWayPointsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dClearWayPointsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int p3dClearWayPointsRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS p3dClearCurrentWayPointRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dClearCurrentWayPointReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int p3dClearCurrentWayPointRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS p3dTrackRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3DSDIIo *(in_trackP),
					 int replyTimeOut);

extern int p3dTrackReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dTrackRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3DSDIIo *(in_trackP), 
					
					int *activity, int *bilan);

extern int p3dTrackRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3DSDIIo *(in_trackP), 
					
					int *activity,
					int *bilan);




extern STATUS p3dTrackStopRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int p3dTrackStopReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dTrackStopRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					
					int *activity, int *bilan);

extern int p3dTrackStopRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					
					int *activity,
					int *bilan);




extern STATUS p3dSetP3dPosterRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 GEN_STRING64 *(in_demP3dPosterName),
					 int replyTimeOut);

int p3dSetP3dPosterReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int p3dSetP3dPosterRqstAndRcv (CLIENT_ID clientId, 
				 GEN_STRING64 *(in_demP3dPosterName), 
				 
				 int *bilan);




extern STATUS p3dReadDemP3dPosterRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GEN_STRING64 *(in_demP3dPosterName),
					 int replyTimeOut);

extern int p3dReadDemP3dPosterReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dReadDemP3dPosterRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GEN_STRING64 *(in_demP3dPosterName), 
					
					int *activity, int *bilan);

extern int p3dReadDemP3dPosterRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GEN_STRING64 *(in_demP3dPosterName), 
					
					int *activity,
					int *bilan);




extern STATUS p3dSaveP3dMapRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3D_SAVEMAP_PARAMS *(in_p3dSaveMapParams),
					 int replyTimeOut);

extern int p3dSaveP3dMapReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dSaveP3dMapRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3D_SAVEMAP_PARAMS *(in_p3dSaveMapParams), 
					
					int *activity, int *bilan);

extern int p3dSaveP3dMapRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3D_SAVEMAP_PARAMS *(in_p3dSaveMapParams), 
					
					int *activity,
					int *bilan);




extern STATUS p3dPlaceRobotOnMapRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3D_PLACEROBOT_PARAMS *(in_placeRobotPos),
					 int replyTimeOut);

extern int p3dPlaceRobotOnMapReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dPlaceRobotOnMapRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3D_PLACEROBOT_PARAMS *(in_placeRobotPos), 
					
					int *activity, int *bilan);

extern int p3dPlaceRobotOnMapRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3D_PLACEROBOT_PARAMS *(in_placeRobotPos), 
					
					int *activity,
					int *bilan);




extern STATUS p3dConfigMonitorRqstSend (CLIENT_ID clientId, int *pRqstId,
					 P3D_MONITOR_PARAMS *(in_monitorParams),
					 int replyTimeOut);

extern int p3dConfigMonitorReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int p3dConfigMonitorRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					P3D_MONITOR_PARAMS *(in_monitorParams), 
					
					int *activity, int *bilan);

extern int p3dConfigMonitorRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					P3D_MONITOR_PARAMS *(in_monitorParams), 
					
					int *activity,
					int *bilan);




extern STATUS p3dStopMonitorRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dStopMonitorReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int p3dStopMonitorRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS p3dGetDumpParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int p3dGetDumpParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       P3D_DUMP_PARAMS *(out_dumpParams),
			       int *bilan);

int p3dGetDumpParamsRqstAndRcv (CLIENT_ID clientId, 
				  
				 P3D_DUMP_PARAMS *(out_dumpParams),
				 int *bilan);




extern STATUS p3dSetDumpParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 P3D_DUMP_PARAMS *(in_dumpParams),
					 int replyTimeOut);

int p3dSetDumpParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int p3dSetDumpParamsRqstAndRcv (CLIENT_ID clientId, 
				 P3D_DUMP_PARAMS *(in_dumpParams), 
				 
				 int *bilan);


#ifdef __cplusplus
}
#endif
#endif /* p3d_MSG_LIB_H */

